/*
 * global.c: Some "global variable" definitions that don't fit elsewhere.
 * 
 * Copyright 2006 the Ithildin Project.
 * See the COPYING file for more information on licensing and use.
 * 
 * "In My Egotistical Opinion, most people's C programs should be indented 
 * six feet downward and covered with dirt."  -- Blair P. Houghton
 */

#include <ithildin/ithildin.h>
#ifdef HAVE_OPENSSL
# include <openssl/err.h>
# include <openssl/rand.h>
# include <openssl/ssl.h>
#endif

IDSTRING(rcsid, "$Id$");

/* depending on the platform we might need to declare _malloc_options.  We
 * don't need to do this for FreeBSD since it's declared in stdlib.h for us
 * (handy) but we do for any other systems (I guess?) */
#ifndef __FreeBSD__
# ifdef MALLOC_OVERRIDE
extern
# endif
const char *_malloc_options;
#endif

/* Our descriptive structure. */
struct me_t me;

HOOK_FUNCTION(reload_conf) {
    char currdir[PATH_MAX];
    conf_list_t *newconf = NULL;
    conf_list_t *oldconf = NULL;

    getcwd(currdir, PATH_MAX);
    if (chdir(me.conf_path)) {
        log_error("couldn't chdir to conf directory %s while reloading conf ",
                me.conf_path);
        return NULL;
    }
    newconf = read_conf(me.conf_file);
    if (newconf == NULL) {
        log_error("couldn't read configuration file %s while reloading conf!",
                me.conf_file);
        return NULL;
    }
    chdir(currdir);

    oldconf = me.confhead;
    me.confhead = newconf;
    build_module_list(); /* rebuild the module list */
    hook_event(me.events.read_conf, (void *)me.confhead);
    destroy_conf_branch(oldconf);
    log_notice("reloaded configuration file %s/%s", me.conf_path,
            me.conf_file);

    return NULL;
}

/* eventually aborting existence should consist of more cleanup, right now
 * all we do is destroy all our sockets.  probably what I should do is write
 * a *_shutdown function for each subsystem.  later.  */
HOOK_FUNCTION(exit_process) {
    struct isocket *sp;

    /* if this function is called pre-shutdown, simply set the shutdown flag,
     * we will get called again later to do cleanup. */
    if (me.shutdown == 0) {
        me.shutdown = -1;
        return NULL;
    }

    /* unload all our modules */
    unload_all_modules();

    /* free our conf stuff */
    destroy_conf_branch(me.confhead);

    /* thrash all our sockets */
    reap_dead_sockets();
    LIST_FOREACH(sp, &allsockets, intlp) {
        destroy_socket(sp);
    }
    reap_dead_sockets();

#ifdef HAVE_OPENSSL
    if (me.ssl.enabled)
        ERR_free_strings();
#endif

    exit(0);
    return NULL; /* NOTREACHED */
}

/* vi:set ts=8 sts=4 sw=4 tw=76 et: */
