/*
 * echo.c: echo module code
 * 
 * Copyright 2006 the Ithildin Project.
 * See the COPYING file for more information on licensing and use.
 * 
 * I wrote this as a new sample module for the 1.2 release to demonstrate
 * the new build system (at least, the GNU toolchain end).  It is a generic
 * implementation of the "echo" service, which reads data you send it and
 * writes that data back.  This is a preposterously useless service.
 * 
 */

#include <ithildin/ithildin.h>

IDSTRING(rcsid, "$Id$");

MODULE_REGISTER("$Rev$");

HOOK_FUNCTION(echo_listener_hook);
HOOK_FUNCTION(echo_socket_hook);

isocket_t *listensocket;

MODULE_LOADER(echo) {
    conf_list_t *conf = *confdata;
    char *s;
    char *port = "10007";
    char *addr = (char *)"0.0.0.0"; /* any address */
    int gsa_port;
    char gsa_host[FQDN_MAXLEN + 1];

    if ((s = conf_find_entry("port", conf, 1)) != NULL)
        port = s;
    if ((s = conf_find_entry("address", conf, 1)) != NULL)
        addr = s;

    listensocket = create_socket();
    if (listensocket == NULL) {
        log_error("couldn't create listener");
        return 0;
    }
    if (!set_socket_address(isock_laddr(listensocket), addr, port,
                SOCK_STREAM)) {
        log_error("couldn't set address/port %s/%s for listener", addr, port);
        return 0;
    }
    if (!open_socket(listensocket)) {
        log_error("couldn't open listener");
        return 0;
    }
    if (!socket_listen(listensocket)) {
        log_error("couldn't listen on listener!");
        return 0;
    }

    /* well with that all done, we set our handler and let the server wait
     * for activity. */
    add_hook(listensocket->datahook, echo_listener_hook);
    socket_monitor(listensocket, SOCKET_FL_READ);

    get_socket_address(isock_laddr(listensocket), gsa_host, FQDN_MAXLEN + 1,
            &gsa_port);
    log_notice("listening for activity on %s/%d", gsa_host, gsa_port);

    return 1;
}

MODULE_UNLOADER(echo) {

    destroy_socket(listensocket);
}

HOOK_FUNCTION(echo_listener_hook) {
    isocket_t *isp;

    while ((isp = socket_accept(listensocket)) != NULL) {
        add_hook(isp->datahook, echo_socket_hook);
        socket_monitor(isp, SOCKET_FL_READ);
    }

    return NULL;
}

HOOK_FUNCTION(echo_socket_hook) {
    isocket_t *isp = (isocket_t *)data;
    int res = 0;

    if (SOCKET_ERROR(isp)) {
        destroy_socket(isp);
        return NULL;
    }
    while (SOCKET_READ(isp)) {
        char rbuf[1];
        /* Read byte-at-a-time and write back. */
        while ((res = socket_read(isp, rbuf, 1)) == 1)
            socket_write(isp, rbuf, 1); /* return value not interesting. */
        if (res < 0)
            destroy_socket(isp); /* not interested in why it failed */
    }

    return NULL;
}

/* vi:set ts=8 sts=4 sw=4 tw=76 et: */
