/*
 * private.c: Flag mode for 'private' channels.
 * 
 * Copyright 2002-2007 the Ithildin Project.
 * See the COPYING file for more information on licensing and use.
 * 
 * This is just a dummy module for networks which still want to use the +p mode
 * in the rfc1459(ish) way.  +p is basically a duplicate of +s, and really
 * there's no reason to use it except to satisfy legacy
 */

#include <ithildin/ithildin.h>

#include "ircd.h"

IDSTRING(rcsid, "$Id$");

MODULE_REGISTER("$Rev$");
const char *mdepends[] = MDEPENDS;

static unsigned char chanmode_private;
static HOOK_FUNCTION(can_show_chan_private);

MODULE_LOADER(private) {

    if (!get_module_savedata(savelist, "chanmode_private",
                &chanmode_private))
        chanmode_request('p', &chanmode_private, CHANMODE_FL_D,
                chanmode_flag, chanmode_flag_query, 0, NULL);
    else
        chanmode_update_funcs(chanmode_private, chanmode_flag,
                chanmode_flag_query);

    add_hook(ircd.events.can_see_channel, can_show_chan_private);

    return 1;
}

MODULE_UNLOADER(private) {
    
    remove_hook(ircd.events.can_see_channel, can_show_chan_private);

    if (reload)
        add_module_savedata(savelist, "chanmode_private",
                sizeof(chanmode_private), &chanmode_private);
    else
        chanmode_release(chanmode_private);
}

HOOK_FUNCTION(can_show_chan_private) {
    struct channel_check_args *ccap = (struct channel_check_args *)data;

    if (ccap->clp != NULL)
        return (void *)HOOK_COND_OK;
    else if (chanmode_isset(ccap->chan, chanmode_private)) {
        if (CLIENT_OPERATOR(ccap->cli) &&
                BPRIV(ccap->cli, ircd.privileges.priv_shc))
            return (void *)HOOK_COND_ALWAYSOK; /* okay, but sketchy.  */
        else
            return (void *)ERR_NOTONCHANNEL;
    } else
        return (void *)HOOK_COND_OK; /* it's not +s, okay by us. */
}

/* vi:set ts=8 sts=4 sw=4 tw=76 et: */
