/*
 * connection.h: support structures/prototypes for connection.c
 * 
 * Copyright 2002-2006 the Ithildin Project.
 * See the COPYING file for more information on licensing and use.
 * 
 * $Id$
 */

#ifndef IRCD_CONNECTION_H
#define IRCD_CONNECTION_H

struct connection {
    isocket_t *sock;                /* our socket */
    client_t *cli;                  /* either a client or a server, should */
    server_t *srv;                  /* never be both! */
    char    host[HOSTLEN + 1];      /* user and host data (as given by
                                       identd/dns) */
    char    user[USERLEN + 1];
    char    *pass;                  /* password, if not NULL it must be
                                       malloc'd memory.  It will be
                                       destroyed after client registration.  */

    char    *buf;                   /* temporary buffer pointer, possibly
                                       used in many places */
    size_t  buflen;                 /* length of data in the buffer */
    size_t  bufsize;                /* size of the temporary buffer. */
    size_t  bufresize;              /* When non-0 a resize request for the
                                       buffer.  We do this in the socket
                                       event hook, after reading has
                                       completed. */

    class_t   *cls;                 /* our connection class */
    protocol_t *proto;              /* our current protocol */
    message_set_t *mset;            /* our message set, NULL for servers */

    time_t  signon;                 /* time of connection (not registration) */
    time_t  last;                   /* last update to this item */
    int     flood;                  /* flood level (clients only) */

    struct {
        int64_t sent;               /* bytes sent */
        int64_t psent;              /* "packets" sent */
        int64_t recv;               /* bytes received */
        int64_t precv;              /* "packets" received */
    } stats;

#define IRCD_CONNFL_WRITEABLE 0x100 /* set if the socket is writeable */
#define CONN_PINGSENT(conn) (conn->flags & IRCD_CONNFL_PINGSENT)
#define IRCD_CONNFL_PINGSENT  0x200 /* set when a PING is sent to test for
                                       activity */
        
#define IRCD_CONNFL_NOSENDQ   0x400 /* this is set if we don't want to observe
                                       send queue limits on the connection
                                       (mostly useful for sending
                                       synchronization data to servers).  it is
                                       not unset until sendq drops to 0. */
#ifdef HAVE_OPENSSL
#define IRCD_CONNFL_SSLINIT   0x800 /* set when we're waiting for the first
                                       data event for an initiating SSL socket.
                                       when we get this event we can begin the
                                       regular connection initiation. */
#endif

    int     flags;                  /* connection flags (DO NOT PUT
                                       CLIENT/SERVER FLAGS HERE) */

    int     sendq_items;            /* items on the send queue */
    STAILQ_HEAD(, sendq_item) sendq;/* and te queue itself */
    LIST_ENTRY(connection) lp;
};

bool connection_set_unregistered(connection_t *);
bool connection_set_registered(connection_t *);

void destroy_connection(connection_t *, char *);
int close_unknown_connections(char *);
int sendq_flush(connection_t *);

HOOK_FUNCTION(ircd_connection_datahook);
HOOK_FUNCTION(ircd_writer_hook);

#endif
/* vi:set ts=8 sts=4 sw=4 tw=76 et: */
