/*
 * compare.c: SuperString comparison function
 * 
 * Copyright 2007 Chip Norkus.
 * See the COPYING file for more information on licensing and use.
 * 
 */

#include <superstring/superstring.h>
#include "superstring_private.h"

IDSTRING(rcsid, "$Id$");

int ss_compare_ex(superstring lstr, superstring rstr,
        int flags, size_t length) {
    ss_internal l, r;
    const unsigned char *table;

    if (lstr == NULL || rstr == NULL)
        return SS_ERROR_INVALID_PARAMETER;

    ss_to_ssi(lstr, &l);
    /* rstr may be a C string */
    if (flags & SS_COMPARE_CSTRING)
        cstr_to_ssi((const unsigned char *)rstr, &r);
    else
        ss_to_ssi(rstr, &r);

    /* If we weren't given a length we can short-circuit equality tests
     * here, or get a length to work with. */
    if (length == 0) {
        if (l.len > r.len)
            return 1;
        else if (l.len < r.len)
            return -1;
        else
            length = l.len;
    }

    if (flags & SS_COMPARE_IGNORECASE)
        table = ss_ascii_lowertab;
    else
        table = ss_ascii_tab;

    /* we are not going to re-convert l/r so we can fiddle with their values
     * indiscriminately below. */
    while (length) {
        /* Check to see if we ran out of data on either string */
        if (r.len == 0) {
            if (l.len == 0)
                return 0; /* they are equal */
            return 1; /* still have data in l, l must be larger */
        } else if (l.len == 0)
            return -1; /* still have data in r, l must be smaller. */

        if (table[*l.data] > table[*r.data])
            return 1;
        else if (table[*l.data] < table[*r.data])
            return -1;

        l.data++;
        l.len--;
        r.data++;
        r.len--;
        length--;
    }

    /* if we made it out here the strings are equal */
    return 0;
}

/* vi:set ts=8 sts=4 sw=4 tw=76 et: */
